#include <stdio.h>   /* Standard input/output definitions */
#include <string.h>  /* String function definitions */
#include <unistd.h>  /* UNIX standard function definitions */
#include <fcntl.h>   /* File control definitions */
#include <errno.h>   /* Error number definitions */
#include <termios.h> /* POSIX terminal control definitions */
#include <ctype.h>   /* isprint */

#define MAYOR 1
#define MINOR 10

struct termios options;


/*
 *      * 'open_port()' - Open serial port ttyUSB0.
 *      *
 *      * Returns the file descriptor on success or -1 on error.
 *                     
 */

int info(void)
{
    printf("\nUnix progswitch, written by Daniel Kotschate, \ncontrols mysmartusb provided by myavr.\n");
    printf("For more information, visit http://www.myavr.de or \nhttp://www.epyx-online.de\n\n");
    printf("Usage :\n");
    printf("\tprogswitch -Option -i/dev/ttyXY\n\n");
    printf("Options :\n");
    printf("\t-d : datamode\n");
    printf("\t-p : programmer mode\n");
    printf("\t-i : tty-dev\n");
    printf("\t-r : reset board\n");
    printf("\t-R : Reset programmer\n");
    printf("\t-v : version\n\n");
    printf("Example : \n\tprogswitch -d -i/dev/ttyUSB0 turns ttyUSB0 into data mode\n\n");
    return 0;
}



int open_port(char* tty)
{
    int fd; /* File descriptor for the port */

    fd = open(tty, O_RDWR | O_NOCTTY | O_NDELAY);
    if (fd == -1)
    {
        return -1;
    }
    else
    {
        fcntl(fd, F_SETFL, 0);
        return (fd);
    }
}

int configure_port(int device)
{
    tcgetattr(device, &options);

    cfsetispeed(&options, B19200);
    cfsetospeed(&options, B19200);


    /* set raw input, 1 second timeout */
    options.c_cflag     |= (CLOCAL | CREAD);
    options.c_lflag     &= ~(ICANON | ECHO | ECHOE | ISIG);
    options.c_oflag     &= ~OPOST;
    options.c_cc[VMIN]  = 0;
    options.c_cc[VTIME] = 10;

    /* set the options */
    tcsetattr(device, TCSANOW, &options);
    return 0;
}

int write_port(int device, unsigned char mode )
{
    // controlsequence
    unsigned char controlseq[] = {0xE6, 0xB5, 0xBA, 0xB9, 0xB2, 0xB3, 0xA9, 0x69,'\n'};

    if (mode > 0 ) controlseq[7] = mode;

    char buffer[255];  /* Input buffer */
    char *bufptr;      /* Current char in buffer */
    int  nbytes;       /* Number of bytes read */ 

    /* send an command*/
    write(device, controlseq, 8);

    /* read characters into our string buffer until we get a CR or NL */
    bufptr = buffer;
    while ((nbytes = read(device, bufptr, buffer + sizeof(buffer) - bufptr - 1)) > 0)
    {
        bufptr += nbytes;
        if (bufptr[-1] == 0x71 || bufptr[-1] == 0xBF)
            break;
    }
    /* nul terminate the string and see if we got an OK response */
    *bufptr = '\0';
    return 0;   	
}


int close_port(int device)
{
    close(device);
    return 0;
}

int main(int argc, char *argv[])
{
    //printf("\33[2J");
    int device = 0;
    unsigned char mode = 0;
    int	commandflag = 0;
    char *ttydev = NULL;
    int c = 0;
    extern char *optarg;
    extern int optind, optopt;

    while ((c = getopt (argc, argv, "pdrRvi:")) != -1)
        switch (c)
        {
            case 'p':
                mode = 0x70;
                break;
            case 'd':
                mode = 0x64;
                break;
            case 'r':
                mode = 0x72;
                break;
            case 'R':
                mode = 0x52;
                break;
            case 'i':
                ttydev = optarg;
                commandflag = 1;
                break;
            case 'v':
                printf ( "program version : %d.%d\n", MAYOR, MINOR);
                commandflag = 2;
                break;
            case '?':
                if (optopt == 'i')
                    printf ("option -%c requires an argument, eg an interface.\n", optopt);
                else if (isprint (optopt))
                    printf ( "Unknown option `-%c'.\n", optopt);
                else
                    printf ( "Unknown option character `\\x%x'.\n",optopt);
                return 1;
            default:
                break; 
        }


    if (commandflag == 1)
    {	
        device = open_port(ttydev);
        if (device > 0) 
        {
            configure_port(device);	
            write_port(device, mode);
            close_port(device);
            printf("\nREADY ! mode %c set to %s \n\n", mode, ttydev);
        }
        else
        {
            printf("\nUnable to open %s!\n\n", ttydev);
        }
    }
    else if(commandflag == 2 ){

    }
    else
    {
        info();
    }
    return 0;
}
